﻿using GrxCAD.ApplicationServices;
using GrxCAD.DatabaseServices;
using GrxCAD.EditorInput;
using GrxCAD.Windows;
using GrxCAD.Runtime;

[assembly: CommandClass(typeof(Sample_CS.SampleCmd))]

namespace Sample_CS
{
	public class SampleCmd : IExtensionApplication
	{
		//パレットとパレットセットはアセンブリファイルロード中は保持
		public static SamplePalette m_SamplePalette;
		public static PaletteSet m_PaletteSet;

		//ドキュメントコレクション
		public DocumentCollection Documents { get { return Application.DocumentManager; } }
		//アクティブなドキュメント
		public Document ActiveDocument { get { return Documents.MdiActiveDocument; } }
		//アクティブなデータベース
		public Database ActiveDatabase { get { return ActiveDocument.Database; } }
		//アクティブなエディタ
		public Editor ActiveEditor { get { return ActiveDocument.Editor; } }

		//初期化処理
		public void Initialize()
		{
			m_SamplePalette = null;
			m_PaletteSet = null;

			//イベントを追加
			Documents.DocumentCreated += new DocumentCollectionEventHandler(DocColDocumentCreated);
			Documents.DocumentToBeDestroyed += new DocumentCollectionEventHandler(DocColDocumentToBeDestroyed);
			Documents.DocumentToBeDeactivated += new DocumentCollectionEventHandler(DocColDocumentToBeDeactivated);
			Documents.DocumentActivated += new DocumentCollectionEventHandler(DocColDocumentActivated);

			ActiveEditor.WriteMessage("\nサンプルの.NETアセンブリをロードしました\n");			
		}
		//解放処理
		public void Terminate()
		{
		}

		//パレットとパレットセットを表示
		[CommandMethod("SAMPLEPALETTE")]
		public void ShowSamplePalette()
		{
			//パレットセットが設定されていない時に実行
			if (m_SamplePalette == null)
			{
				//パレットセットの作成
				m_PaletteSet = new PaletteSet("SAMPLE PALETTESET", new System.Guid("A53C4C86-6D6F-4702-AC54-6E120CF8DD55"));

				//パレットセットのスタイル
				m_PaletteSet.Style =
					PaletteSetStyles.ShowTabForSingle |
					PaletteSetStyles.ShowPropertiesMenu |
					PaletteSetStyles.ShowAutoHideButton |
					PaletteSetStyles.ShowCloseButton |
					PaletteSetStyles.Snappable;

				//パレットセットの最小サイズ
				m_PaletteSet.MinimumSize = new System.Drawing.Size(240, 160);
				//パレットセットのサイズ
				m_PaletteSet.Size = new System.Drawing.Size(300, 160);
				//パレットセットを表示
				m_PaletteSet.Visible = true;
			}
			else
			{
				//パレットセットを表示
				m_PaletteSet.Visible = true;
			}

			//パレットセットに項目が無い場合パレットを追加
			if (m_PaletteSet.Count == 0)
			{
				//パレットを作成
				m_SamplePalette = new SamplePalette();

				//パレットをパレットセットに追加
				m_PaletteSet.Add("SAMPLE PALETTE", m_SamplePalette);
			}

			//画面を更新
			Application.UpdateScreen();

			//ポイントモニターイベントを追加
			ActiveEditor.PointMonitor += new PointMonitorEventHandler(PointMonitor);
		}

		/// <summary>
		/// パレットセットを非表示にする
		/// </summary>
		public void HideSamplePalette()
		{
			if (m_PaletteSet != null)
			{				
				//パレットセットを非表示
				m_PaletteSet.Visible = false;
			}
			//画面を更新
			Application.UpdateScreen();
		}

		/// <summary>
		/// ドキュメント ウィンドウが作成されると発生するイベント
		/// </summary>
		public void DocColDocumentCreated(object senderObj, DocumentCollectionEventArgs e)
		{
			try
			{
				DocumentCollection Docs = senderObj as DocumentCollection;

				if (Docs.Count == 1 && m_PaletteSet != null)
				{
					m_PaletteSet.Visible = true;
					ActiveEditor.PointMonitor += new PointMonitorEventHandler(PointMonitor);
				}
				Application.UpdateScreen();
			}
			catch (GrxCAD.Runtime.Exception ex)
			{
				Application.ShowAlertDialog(ex.Message + ex.StackTrace);
			}
		}
		/// <summary>
		/// ドキュメントが破棄される直前に発生するイベント
		/// </summary>
		public void DocColDocumentToBeDestroyed(object senderObj, DocumentCollectionEventArgs e)
		{
			try
			{
				DocumentCollection Docs = senderObj as DocumentCollection;
				if (Docs.Count == 1)
					HideSamplePalette();
			}
			catch (GrxCAD.Runtime.Exception ex)
			{
				Application.ShowAlertDialog(ex.Message + ex.StackTrace);
			}
		}
		/// <summary>
		/// ドキュメントが非アクティブ化される直前に発生するイベント
		/// </summary>
		public void DocColDocumentToBeDeactivated(object senderObj, DocumentCollectionEventArgs e)
		{
			try
			{
				e.Document.Editor.PointMonitor -= new PointMonitorEventHandler(PointMonitor);
			}
			catch (GrxCAD.Runtime.Exception ex)
			{
				Application.ShowAlertDialog(ex.Message + ex.StackTrace);
			}
		}
		/// <summary>
		/// ドキュメントがアクティブになった時点で発生するイベント
		/// </summary>
		public void DocColDocumentActivated(object senderObj, DocumentCollectionEventArgs e)
		{
			try
			{
				e.Document.Editor.PointMonitor += new PointMonitorEventHandler(PointMonitor);
			}
			catch (GrxCAD.Runtime.Exception ex)
			{
				Application.ShowAlertDialog(ex.Message + ex.StackTrace);
			}
		}
		/// <summary>
		/// ポイントモニターイベント
		/// </summary>
		public void PointMonitor(object senderObj, PointMonitorEventArgs e)
		{
			try
			{
				if (m_PaletteSet != null && m_SamplePalette != null)
				{
					m_SamplePalette.SetCursolPos(e.Context.RawPoint.ToArray());
					m_SamplePalette.ClearList();

					PromptSelectionResult SelRes = ActiveEditor.SelectImplied();
					if (SelRes.Status == PromptStatus.OK)
					{
						using (var Trans = ActiveDatabase.TransactionManager.StartTransaction())
						{
							foreach (SelectedObject SelObj in SelRes.Value)
							{
								Entity tmpEnt = Trans.GetObject(SelObj.ObjectId, OpenMode.ForRead) as Entity;
								m_SamplePalette.SetSelectedObjectsList(tmpEnt.GetRXClass().DxfName);
								tmpEnt.Dispose();
							}
						}
					}
				}
				else
				{
					ActiveEditor.PointMonitor -= new PointMonitorEventHandler(PointMonitor);
				}
			}
			catch (GrxCAD.Runtime.Exception ex)
			{
				Application.ShowAlertDialog(ex.Message + ex.StackTrace);
			}			
		}
	}
}