﻿Imports GrxCAD.ApplicationServices
Imports GrxCAD.DatabaseServices
Imports GrxCAD.EditorInput
Imports GrxCAD.Windows
Imports GrxCAD.Runtime

<Assembly: CommandClass(GetType(Sample_VB.SampleCmd))>

Namespace Sample_VB
	Public Class SampleCmd : Implements IExtensionApplication

		'パレットとパレットセットはアセンブリファイルロード中は保持
		Public Shared m_SamplePalette As SamplePalette
		Public Shared m_PaletteSet As PaletteSet

		'ドキュメントコレクション
		Public ReadOnly Property Documents As DocumentCollection
			Get
				Return Application.DocumentManager
			End Get
		End Property
		'アクティブなドキュメント
		Public ReadOnly Property ActiveDocument As Document
			Get
				Return Documents.MdiActiveDocument
			End Get
		End Property
		'アクティブなデータベース
		Public ReadOnly Property ActiveDatabase As Database
			Get
				Return ActiveDocument.Database
			End Get
		End Property
		'アクティブなエディタ
		Public ReadOnly Property ActiveEditor As Editor
			Get
				Return ActiveDocument.Editor
			End Get
		End Property

		'初期化処理
		Public Sub Initialize() Implements IExtensionApplication.Initialize
			m_SamplePalette = Nothing
			m_PaletteSet = Nothing

			'イベントを追加
			AddHandler Documents.DocumentCreated, AddressOf DocColDocumentCreated
			AddHandler Documents.DocumentToBeDestroyed, AddressOf DocColDocumentToBeDestroyed
			AddHandler Documents.DocumentToBeDeactivated, AddressOf DocColDocumentToBeDeactivated
			AddHandler Documents.DocumentActivated, AddressOf DocColDocumentActivated

			ActiveEditor.WriteMessage("サンプルの.NETアセンブリをロードしました")
		End Sub

        '解放処理
        Public Sub Terminate() Implements IExtensionApplication.Terminate
		End Sub

		'パレットとパレットセットを表示
		<CommandMethod("SAMPLEPALETTE")>
		Public Sub ShowSamplePalette()
			'パレットセットが設定されていない時に実行
			If m_PaletteSet Is Nothing Then
				'パレットセットの作成
				m_PaletteSet = New PaletteSet("SAMPLE PALETTESET", New System.Guid("A53C4C86-6D6F-4702-AC54-6E120CF8DD55"))

				'パレットセットのスタイル
				m_PaletteSet.Style =
					PaletteSetStyles.ShowTabForSingle Or
					PaletteSetStyles.ShowPropertiesMenu Or
					PaletteSetStyles.ShowAutoHideButton Or
					PaletteSetStyles.ShowCloseButton Or
					PaletteSetStyles.Snappable

				'パレットセットの最小サイズ
				m_PaletteSet.MinimumSize = New System.Drawing.Size(240, 160)

				'パレットセットのサイズ
				m_PaletteSet.Size = New System.Drawing.Size(300, 160)

				'パレットセットを表示
				m_PaletteSet.Visible = True
			Else
				'パレットセットを表示
				m_PaletteSet.Visible = True
			End If

			'パレットセットに項目が無い場合パレットを追加
			If m_PaletteSet.Count = 0 Then
				'パレットを作成
				m_SamplePalette = New SamplePalette()

				'パレットをパレットセットに追加
				m_PaletteSet.Add("SAMPLE PALETTE", m_SamplePalette)
			End If

			'画面を更新
			Application.UpdateScreen()
			'ポイントモニターイベントを追加
			AddHandler ActiveEditor.PointMonitor, AddressOf PointMonitor
		End Sub

		''' <summary>
		''' パレットセットを非表示にする
		''' </summary>
		Public Sub HideSamplePalette()
			If m_PaletteSet <> Nothing Then
				'パレットセットを非表示
				m_PaletteSet.Visible = False
			End If
			'画面を更新
			Application.UpdateScreen()
		End Sub

		''' <summary>
		''' ドキュメント ウィンドウが作成されると発生するイベント
		''' </summary>
		Public Sub DocColDocumentCreated(ByVal sender As Object, ByVal e As DocumentCollectionEventArgs)
			Try
				Dim Docs As DocumentCollection = sender

				If Docs.Count = 1 AndAlso m_PaletteSet <> Nothing Then
					m_PaletteSet.Visible = True
					AddHandler ActiveEditor.PointMonitor, AddressOf PointMonitor
				End If
			Catch ex As GrxCAD.Runtime.Exception
				Application.ShowAlertDialog(ex.Message + ex.StackTrace)
			End Try
		End Sub

		''' <summary>
		''' ドキュメントが破棄される直前に発生するイベント
		''' </summary>
		Public Sub DocColDocumentToBeDestroyed(ByVal sender As Object, ByVal e As DocumentCollectionEventArgs)
			Try
				Dim Docs As DocumentCollection = sender
				If Docs.Count = 1 Then
					HideSamplePalette()
				End If
			Catch ex As GrxCAD.Runtime.Exception
				Application.ShowAlertDialog(ex.Message + ex.StackTrace)
			End Try
		End Sub

		''' <summary>
		''' ドキュメントが非アクティブ化される直前に発生するイベント
		''' </summary>
		Public Sub DocColDocumentToBeDeactivated(ByVal sender As Object, ByVal e As DocumentCollectionEventArgs)
			Try
				RemoveHandler e.Document.Editor.PointMonitor, AddressOf PointMonitor
			Catch ex As GrxCAD.Runtime.Exception
				Application.ShowAlertDialog(ex.Message + ex.StackTrace)
			End Try
		End Sub

		''' <summary>
		''' ドキュメントがアクティブになった時点で発生するイベント
		''' </summary>
		Public Sub DocColDocumentActivated(ByVal sender As Object, ByVal e As DocumentCollectionEventArgs)
			Try
				AddHandler e.Document.Editor.PointMonitor, AddressOf PointMonitor
			Catch ex As GrxCAD.Runtime.Exception
				Application.ShowAlertDialog(ex.Message + ex.StackTrace)
			End Try
		End Sub

		''' <summary>
		''' ポイントモニターイベント
		''' </summary>
		Public Sub PointMonitor(ByVal sender As Object, ByVal e As PointMonitorEventArgs)
			Try
				If (m_PaletteSet IsNot Nothing AndAlso m_SamplePalette IsNot Nothing) Then
					m_SamplePalette.SetCursolPos(e.Context.RawPoint.ToArray())
					m_SamplePalette.ClearList()

					Dim SelRes As PromptSelectionResult = ActiveEditor.SelectImplied()
					If SelRes.Status = PromptStatus.OK Then
						Using Trans As Transaction = ActiveDatabase.TransactionManager.StartTransaction()
							For Each SelObj As SelectedObject In SelRes.Value
								Dim tmpEnt As Entity = Trans.GetObject(SelObj.ObjectId, OpenMode.ForRead)
								m_SamplePalette.SetSelectedObjectsList(tmpEnt.GetRXClass().DxfName)
								tmpEnt.Dispose()
							Next
						End Using
					End If
				Else
					RemoveHandler ActiveEditor.PointMonitor, AddressOf PointMonitor
				End If
			Catch ex As GrxCAD.Runtime.Exception
				Application.ShowAlertDialog(ex.Message + ex.StackTrace)
			End Try
		End Sub

	End Class
End Namespace

